/**
 * @file debug.c
 * @brief Funes de depurao
 *
 * Funes de depurao que sero chamadas atravs das respectivas
 * macros definidas no ficheiro debug.h. O objectivo destas funes 
 *  auxiliar o tratamento de erros e a depurao
 *
 * @author Miguel Frade, Patricio Domingues, Vitor Carreira
 * @date Agosto de 2003
 * @version 2 
 */
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <netdb.h>

#include "debug.h"

/**
 * Esta funo deve ser utilizada para auxiliar a depurao de programas.
 * Esta funo <b>no deve</b> ser chamada directamente, mas sim atravs
 * da macro DEBUG().
 *
 * @param file nome do ficheiro
 * 	       (atravs da macro DEBUG)
 * @param line linha onde a funo foi chamada
 * 	       (atravs da macro DEBUG)
 * @param fmt string de formatao como no "printf"
 * @param ... n varivel de parmetros
 * @return A funo no retorna nada
 * @see DEBUG
 */
void debug(const char *file, const int line, char *fmt, ...)
{
    va_list ap;

    va_start(ap, fmt);
    fprintf(stderr, "[%s@%d] DEBUG - ", file, line);
    vfprintf(stderr, fmt, ap);
    va_end(ap);
    fprintf(stderr, "\n");
    fflush(stderr);
}


/**
 * Funo que envia para o canal de erros a mensagem "WARNING" rotulada
 * com o nome do ficheiro e da linha da funo chamante e ainda da mensagem de
 * erro do sistema. A funo <b>no deve</b> ser chamada directamente, mas sim
 * atravs da macro WARNING().
 *
 * @param file nome do ficheiro fonte da funo chamante
 * 	       (atravs da macro WARNING)
 * @param line linha onde a funo foi chamada
 * 	       (atravs da macro WARNING)
 * @param fmt string de formatao como no "printf"
 * @param ... n varivel de parmetros
 * @return A funo no retorna nada
 * @see WARNING
 */
void warning(const char *file, const int line, char *fmt, ...)
{
    va_list ap;

    va_start(ap, fmt);
    fprintf(stderr, "[%s@%d] WARNING - ", file, line);
    vfprintf(stderr, fmt, ap);
    va_end(ap);
	if (errno != 0)
	    fprintf(stderr, ": %s\n", strerror(errno));
    fflush(stderr);
}



/**
 * Funo que envia para o canal de erros a mensagem "ERROR" rotulada
 * com o nome do ficheiro e da linha da funo chamante, e ainda da mensagem de
 * erro do sistema. A funo <b>no deve</b> ser chamada directamente, mas sim
 * atravs da macro ERROR().
 *
 * @param file nome do ficheiro fonte da funo chamante
 * 	       (atravs da macro ERROR)
 * @param line linha onde a funo foi chamada
 * 	       (atravs da macro ERROR)
 * @param exitCode valor passado  funo "exit()"
 * @param fmt string de formatao como no "printf"
 * @param ... n varivel de parmetros
 * @return A funo no retorna nada
 * @see ERROR
 */
void error(const char *file, const int line, int exitCode, char *fmt, ...)
{
    va_list ap;

    va_start(ap, fmt);
    fprintf(stderr, "[%s@%d] ERROR - ", file, line);
    vfprintf(stderr, fmt, ap);
    va_end(ap);
	if (errno != 0)
	    fprintf(stderr, ": %s\n", strerror(errno));
    fflush(stderr);
    exit(exitCode);
}


/**
 * Funo que envia para o canal de erros a mensagem "H_WARNING" rotulada
 * com o nome do ficheiro e da linha da funo chamante, e ainda da mensagem de
 * erro do sistema. A funo <b>no deve</b> ser chamada directamente, mas sim
 * atravs da macro H_WARNING().
 *
 * @param file nome do ficheiro fonte da funo chamante
 * 	       (atravs da macro H_WARNING)
 * @param line linha onde a funo foi chamada
 * 	       (atravs da macro H_WARNING)
 * @param fmt string de formatao como no "printf"
 * @param ... n varivel de parmetros
 * @return A funo no retorna nada
 * @see H_WARNING
 */
void h_warning(const char *file, const int line, char *fmt, ...)
{
    va_list ap;

    va_start(ap, fmt);
    fprintf(stderr, "[%s@%d] H_WARNING - ", file, line);
    vfprintf(stderr, fmt, ap);
    va_end(ap);
	if (h_errno != 0)
	    fprintf(stderr, ": %s\n", hstrerror(h_errno));
    fflush(stderr);
}


/**
 * Funo que envia para o canal de erros a mensagem "H_ERROR" rotulada
 * com o nome do ficheiro e da linha da funo chamante, e ainda da mensagem de
 * erro do sistema. A funo <b>no deve</b> ser chamada directamente, mas sim
 * atravs da macro H_ERROR().
 *
 * @param file nome do ficheiro fonte da funo chamante 
 * 	       (atravs da macro H_ERROR)
 * @param line linha onde a funo foi chamada
 * 	       (atravs da macro H_ERROR)
 * @param exitCode valor passado  funo "exit()"
 * @param fmt string de formatao como no "printf"
 * @param ... n varivel de parmetros
 * @return A funo no retorna nada
 * @see H_ERROR
 */
void h_error(const char *file, const int line, int exitCode, char *fmt,
	     ...)
{
    va_list ap;

    va_start(ap, fmt);
    fprintf(stderr, "[%s@%d] H_ERROR - ", file, line);
    vfprintf(stderr, fmt, ap);
    va_end(ap);
	if (h_errno != 0)
	    fprintf(stderr, ": %s\n", hstrerror(h_errno));
    fflush(stderr);
    exit(exitCode);
}
